@@ -18,7 +18,7 @@ func (c *initCmd) showByDefault() bool {
 
 func (c *initCmd) usage() string {
 	return gettext.Gettext(
-		"lxc init <image> [<name>] [--ephemeral|-e] [--profile|-p <profile>...]\n" +
+		"lxc init [remote:]<image> [remote:][<name>] [--ephemeral|-e] [--profile|-p <profile>...]\n" +
 			"\n" +
 			"Initializes a container using the specified image and name.\n" +
 			"\n" +
