@@ -2,7 +2,6 @@ package main
 
 import (
 	"fmt"
-	"log"
 	"net"
 	"net/url"
 	"os"
@@ -56,10 +55,15 @@ func run() error {
 		os.Args = append(os.Args[:1], os.Args[2:]...)
 	}
 
-	if len(os.Args) == 2 && (os.Args[1] == "-h" || os.Args[1] == "--help") {
+	if len(os.Args) >= 2 && (os.Args[1] == "-h" || os.Args[1] == "--help") {
 		os.Args[1] = "help"
 	}
 
+	if len(os.Args) >= 2 && (os.Args[1] == "--all") {
+		os.Args[1] = "help"
+		os.Args = append(os.Args, "--all")
+	}
+
 	if len(os.Args) == 2 && os.Args[1] == "--version" {
 		os.Args[1] = "version"
 	}
@@ -84,10 +88,7 @@ func run() error {
 	os.Args = os.Args[1:]
 	gnuflag.Parse(true)
 
-	if *verbose || *debug {
-		shared.SetLogger(log.New(os.Stderr, "", log.LstdFlags))
-		shared.SetDebug(*debug)
-	}
+	shared.SetLogger("", "", *verbose, *debug)
 
 	config, err := lxd.LoadConfig()
 	if err != nil {
@@ -97,10 +98,7 @@ func run() error {
 	certf := lxd.ConfigPath("client.crt")
 	keyf := lxd.ConfigPath("client.key")
 
-	_, err = os.Stat(certf)
-	_, err2 := os.Stat(keyf)
-
-	if os.Args[0] != "help" && os.Args[0] != "version" && (err != nil || err2 != nil) {
+	if os.Args[0] != "help" && os.Args[0] != "version" && (!shared.PathExists(certf) || !shared.PathExists(keyf)) {
 		fmt.Fprintf(os.Stderr, gettext.Gettext("Generating a client certificate. This may take a minute...\n"))
 
 		err = shared.FindOrGenCert(certf, keyf)
@@ -142,6 +140,7 @@ var commands = map[string]command{
 	"list":     &listCmd{},
 	"move":     &moveCmd{},
 	"profile":  &profileCmd{},
+	"publish":  &publishCmd{},
 	"remote":   &remoteCmd{},
 	"restart":  &actionCmd{shared.Restart, true},
 	"restore":  &restoreCmd{},
