@@ -43,22 +43,11 @@ func api10Get(d *Daemon, r *http.Request) Response {
 	if d.isTrustedClient(r) {
 		body["auth"] = "trusted"
 
-		uname := syscall.Utsname{}
-		if err := syscall.Uname(&uname); err != nil {
-			return InternalError(err)
-		}
-
-		backing_fs, err := shared.GetFilesystem(d.lxcpath)
+		backingFs, err := shared.GetFilesystem(d.lxcpath)
 		if err != nil {
 			return InternalError(err)
 		}
 
-		env := shared.Jmap{
-			"lxc_version": lxc.Version(),
-			"lxd_version": shared.Version,
-			"driver":      "lxc",
-			"backing_fs":  backing_fs}
-
 		/*
 		 * Based on: https://groups.google.com/forum/#!topic/golang-nuts/Jel8Bb-YwX8
 		 * there is really no better way to do this, which is
@@ -66,6 +55,19 @@ func api10Get(d *Daemon, r *http.Request) Response {
 		 * version in that thread, since it doesn't seem as portable,
 		 * viz. github issue #206.
 		 */
+		uname := syscall.Utsname{}
+		if err := syscall.Uname(&uname); err != nil {
+			return InternalError(err)
+		}
+
+		kernel := ""
+		for _, c := range uname.Sysname {
+			if c == 0 {
+				break
+			}
+			kernel += string(byte(c))
+		}
+
 		kernelVersion := ""
 		for _, c := range uname.Release {
 			if c == 0 {
@@ -74,7 +76,24 @@ func api10Get(d *Daemon, r *http.Request) Response {
 			kernelVersion += string(byte(c))
 		}
 
-		env["kernel_version"] = kernelVersion
+		kernelArchitecture := ""
+		for _, c := range uname.Machine {
+			if c == 0 {
+				break
+			}
+			kernelArchitecture += string(byte(c))
+		}
+
+		env := shared.Jmap{
+			"architectures":       d.architectures,
+			"backing_fs":          backingFs,
+			"driver":              "lxc",
+			"driver_version":      lxc.Version(),
+			"kernel":              kernel,
+			"kernel_architecture": kernelArchitecture,
+			"kernel_version":      kernelVersion,
+			"version":             shared.Version}
+
 		body["environment"] = env
 
 		serverConfig, err := getServerConfig(d)
@@ -112,18 +131,30 @@ func api10Put(d *Daemon, r *http.Request) Response {
 	}
 
 	for key, value := range req.Config {
+		if !ValidServerConfigKey(key) {
+			return BadRequest(fmt.Errorf("Bad server config key: '%s'", key))
+		}
+
 		if key == "core.trust_password" {
 			err := setTrustPassword(d, value.(string))
 			if err != nil {
 				return InternalError(err)
 			}
-		} else if ValidServerConfigKey(key) {
-			err := setServerConfig(d, key, value.(string))
+		} else if key == "core.lvm_vg_name" {
+			err := setLVMVolumeGroupNameConfig(d, value.(string))
+			if err != nil {
+				return InternalError(err)
+			}
+		} else if key == "core.lvm_thinpool_name" {
+			err := setLVMThinPoolNameConfig(d, value.(string))
 			if err != nil {
 				return InternalError(err)
 			}
 		} else {
-			return BadRequest(fmt.Errorf("Bad server config key: '%s'", key))
+			err := setServerConfig(d, key, value.(string))
+			if err != nil {
+				return InternalError(err)
+			}
 		}
 	}
 
