@@ -3,38 +3,42 @@ package main
 import (
 	"database/sql"
 	"encoding/hex"
-	"errors"
 	"fmt"
 	"os"
 	"time"
 
-	_ "github.com/mattn/go-sqlite3"
+	"github.com/mattn/go-sqlite3"
 
 	"github.com/lxc/lxd/shared"
 )
 
 var (
+	// DbErrAlreadyDefined hapens when the given entry already exists,
+	// for example a container.
 	DbErrAlreadyDefined = fmt.Errorf("already exists")
 
-	/*
-	 * n.b. in the case of joins (and probably other) queries, we don't
-	 * get back sql.ErrNoRows when no rows are returned, even though we do
+	/* NoSuchObjectError is in the case of joins (and probably other) queries,
+	 * we don't get back sql.ErrNoRows when no rows are returned, even though we do
 	 * on selects without joins. Instead, you can use this error to
 	 * propagate up and generate proper 404s to the client when something
 	 * isn't found so we don't abuse sql.ErrNoRows any more than we
 	 * already do.
 	 */
-	NoSuchObjectError = errors.New("No such object")
+	NoSuchObjectError = fmt.Errorf("No such object")
 )
 
+// Profile is here to order Profiles.
 type Profile struct {
 	name  string
 	order int
 }
+
+// Profiles will contain a list of all Profiles.
 type Profiles []Profile
 
-const DB_CURRENT_VERSION int = 8
+const DB_CURRENT_VERSION int = 9
 
+// CURRENT_SCHEMA contains the current SQLite SQL Schema.
 const CURRENT_SCHEMA string = `
 CREATE TABLE IF NOT EXISTS certificates (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -166,12 +170,12 @@ func createDb(db *sql.DB) (err error) {
 
 	// To make the schema creation indempotent, only insert the schema version
 	// if there isn't one already.
-	latest_version := getSchema(db)
+	latestVersion := getSchema(db)
 
-	if latest_version == 0 {
+	if latestVersion == 0 {
 		// There isn't an entry for schema version, let's put it in.
-		insert_stmt := `INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
-		_, err = db.Exec(insert_stmt, DB_CURRENT_VERSION)
+		insertStmt := `INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
+		_, err = db.Exec(insertStmt, DB_CURRENT_VERSION)
 		if err != nil {
 			return err
 		}
@@ -186,13 +190,21 @@ func getSchema(db *sql.DB) (v int) {
 	arg1 := []interface{}{}
 	arg2 := []interface{}{&v}
 	q := "SELECT max(version) FROM schema"
-	err := shared.DbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(db, q, arg1, arg2)
 	if err != nil {
 		return 0
 	}
 	return v
 }
 
+func updateFromV8(db *sql.DB) error {
+	stmt := `
+UPDATE certificates SET fingerprint = replace(fingerprint, " ", "");
+INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
+	_, err := db.Exec(stmt, 9)
+	return err
+}
+
 func updateFromV7(db *sql.DB) error {
 	stmt := `
 UPDATE config SET key='core.trust_password' WHERE key IN ('password', 'trust_password', 'trust-password', 'core.trust-password');
@@ -208,7 +220,7 @@ func updateFromV6(db *sql.DB) error {
 	stmt := `
 PRAGMA foreign_keys=OFF; -- So that integrity doesn't get in the way for now
 
-CREATE TEMP TABLE tmp AS SELECT * FROM containers_config; 
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_config;
 DROP TABLE containers_config;
 CREATE TABLE IF NOT EXISTS containers_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -221,7 +233,7 @@ CREATE TABLE IF NOT EXISTS containers_config (
 INSERT INTO containers_config SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices; 
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices;
 DROP TABLE containers_devices;
 CREATE TABLE IF NOT EXISTS containers_devices (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
@@ -234,7 +246,7 @@ CREATE TABLE IF NOT EXISTS containers_devices (
 INSERT INTO containers_devices SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices_config; 
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices_config;
 DROP TABLE containers_devices_config;
 CREATE TABLE IF NOT EXISTS containers_devices_config (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
@@ -247,7 +259,7 @@ CREATE TABLE IF NOT EXISTS containers_devices_config (
 INSERT INTO containers_devices_config SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM containers_profiles; 
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_profiles;
 DROP TABLE containers_profiles;
 CREATE TABLE IF NOT EXISTS containers_profiles (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
@@ -261,7 +273,7 @@ CREATE TABLE IF NOT EXISTS containers_profiles (
 INSERT INTO containers_profiles SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM images_aliases; 
+CREATE TEMP TABLE tmp AS SELECT * FROM images_aliases;
 DROP TABLE images_aliases;
 CREATE TABLE IF NOT EXISTS images_aliases (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -274,7 +286,7 @@ CREATE TABLE IF NOT EXISTS images_aliases (
 INSERT INTO images_aliases SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM images_properties; 
+CREATE TEMP TABLE tmp AS SELECT * FROM images_properties;
 DROP TABLE images_properties;
 CREATE TABLE IF NOT EXISTS images_properties (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -287,7 +299,7 @@ CREATE TABLE IF NOT EXISTS images_properties (
 INSERT INTO images_properties SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM profiles_config; 
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_config;
 DROP TABLE profiles_config;
 CREATE TABLE IF NOT EXISTS profiles_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -300,7 +312,7 @@ CREATE TABLE IF NOT EXISTS profiles_config (
 INSERT INTO profiles_config SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices; 
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices;
 DROP TABLE profiles_devices;
 CREATE TABLE IF NOT EXISTS profiles_devices (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -313,7 +325,7 @@ CREATE TABLE IF NOT EXISTS profiles_devices (
 INSERT INTO profiles_devices SELECT * FROM tmp;
 DROP TABLE tmp;
 
-CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices_config; 
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices_config;
 DROP TABLE profiles_devices_config;
 CREATE TABLE IF NOT EXISTS profiles_devices_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -390,7 +402,7 @@ INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
 
 	passfname := shared.VarPath("adminpwd")
 	passOut, err := os.Open(passfname)
-	old_password := ""
+	oldPassword := ""
 	if err == nil {
 		defer passOut.Close()
 		buff := make([]byte, 96)
@@ -399,10 +411,10 @@ INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
 			return err
 		}
 
-		old_password = hex.EncodeToString(buff)
+		oldPassword = hex.EncodeToString(buff)
 		stmt := `INSERT INTO config (key, value) VALUES ("core.trust_password", ?);`
 
-		_, err := db.Exec(stmt, old_password)
+		_, err := db.Exec(stmt, oldPassword)
 		if err != nil {
 			return err
 		}
@@ -514,84 +526,90 @@ INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
 	return err
 }
 
-func updateDb(db *sql.DB, prev_version int) error {
-	if prev_version < 0 || prev_version > DB_CURRENT_VERSION {
-		return fmt.Errorf("Bad database version: %d\n", prev_version)
+func updateDb(db *sql.DB, prevVersion int) error {
+	if prevVersion < 0 || prevVersion > DB_CURRENT_VERSION {
+		return fmt.Errorf("Bad database version: %d\n", prevVersion)
 	}
-	if prev_version == DB_CURRENT_VERSION {
+	if prevVersion == DB_CURRENT_VERSION {
 		return nil
 	}
 	var err error
-	if prev_version < 1 {
+	if prevVersion < 1 {
 		err = updateFromV0(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 2 {
+	if prevVersion < 2 {
 		err = updateFromV1(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 3 {
+	if prevVersion < 3 {
 		err = updateFromV2(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 4 {
+	if prevVersion < 4 {
 		err = updateFromV3(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 5 {
+	if prevVersion < 5 {
 		err = updateFromV4(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 6 {
+	if prevVersion < 6 {
 		err = updateFromV5(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 7 {
+	if prevVersion < 7 {
 		err = updateFromV6(db)
 		if err != nil {
 			return err
 		}
 	}
-	if prev_version < 8 {
+	if prevVersion < 8 {
 		err = updateFromV7(db)
 		if err != nil {
 			return err
 		}
 	}
+	if prevVersion < 9 {
+		err = updateFromV8(db)
+		if err != nil {
+			return err
+		}
+	}
 
 	return nil
 }
 
 func createDefaultProfile(db *sql.DB) error {
-	rows, err := shared.DbQuery(db, "SELECT id FROM profiles WHERE name=?", "default")
+	rows, err := dbQuery(db, "SELECT id FROM profiles WHERE name=?", "default")
 	if err != nil {
 		return err
 	}
 	defer rows.Close()
 	id := -1
 	for rows.Next() {
-		var xId int
-		rows.Scan(&xId)
-		id = xId
+		var xID int
+		rows.Scan(&xID)
+		id = xID
 	}
 	if id != -1 {
 		// default profile already exists
 		return nil
 	}
 
-	tx, err := shared.DbBegin(db)
+	tx, err := dbBegin(db)
 	if err != nil {
 		return err
 	}
@@ -607,7 +625,7 @@ func createDefaultProfile(db *sql.DB) error {
 	}
 	id = int(id64)
 
-	result, err = tx.Exec(`INSERT INTO profiles_devices 
+	result, err = tx.Exec(`INSERT INTO profiles_devices
 		(profile_id, name, type) VALUES (?, ?, ?)`,
 		id, "eth0", "nic")
 	if err != nil {
@@ -619,11 +637,11 @@ func createDefaultProfile(db *sql.DB) error {
 		tx.Rollback()
 		return err
 	}
-	devId := int(id64)
+	devID := int(id64)
 
 	_, err = tx.Exec(`INSERT INTO profiles_devices_config
 		(profile_device_id, key, value) VALUES (?, ?, ?)`,
-		devId, "nictype", "bridged")
+		devID, "nictype", "bridged")
 	if err != nil {
 		tx.Rollback()
 		return err
@@ -632,7 +650,7 @@ func createDefaultProfile(db *sql.DB) error {
 	/* TODO - analyze system to choose a bridge */
 	_, err = tx.Exec(`INSERT INTO profiles_devices_config
 		(profile_device_id, key, value) VALUES (?, ?, ?)`,
-		devId, "parent", "lxcbr0")
+		devID, "parent", "lxcbr0")
 	if err != nil {
 		tx.Rollback()
 		return err
@@ -689,7 +707,136 @@ func initDb(d *Daemon) (err error) {
 	return err
 }
 
-// Get an ImageBaseInfo object from the database.
+func dbPasswordGet(db *sql.DB) (pwd string, err error) {
+	q := "SELECT value FROM config WHERE key=\"core.trust_password\""
+	value := ""
+	argIn := []interface{}{}
+	argOut := []interface{}{&value}
+	err = dbQueryRowScan(db, q, argIn, argOut)
+
+	if err != nil || value == "" {
+		return "", fmt.Errorf("No password is set")
+	}
+
+	return value, nil
+}
+
+// dbCertInfo is here to pass the certificates content
+// from the database around
+type dbCertInfo struct {
+	ID          int
+	Fingerprint string
+	Type        int
+	Name        string
+	Certificate string
+}
+
+// dbCertsGet returns all certificates from the DB as CertBaseInfo objects.
+func dbCertsGet(db *sql.DB) (certs []*dbCertInfo, err error) {
+	rows, err := dbQuery(
+		db,
+		"SELECT id, fingerprint, type, name, certificate FROM certificates",
+	)
+	if err != nil {
+		return certs, err
+	}
+
+	defer rows.Close()
+
+	for rows.Next() {
+		cert := new(dbCertInfo)
+		rows.Scan(
+			&cert.ID,
+			&cert.Fingerprint,
+			&cert.Type,
+			&cert.Name,
+			&cert.Certificate,
+		)
+		certs = append(certs, cert)
+	}
+
+	return certs, nil
+}
+
+// dbCertGet gets an CertBaseInfo object from the database.
+// The argument fingerprint will be queried with a LIKE query, means you can
+// pass a shortform and will get the full fingerprint.
+// There can never be more than one image with a given fingerprint, as it is
+// enforced by a UNIQUE constraint in the schema.
+func dbCertGet(db *sql.DB, fingerprint string) (cert *dbCertInfo, err error) {
+	cert = new(dbCertInfo)
+
+	inargs := []interface{}{fingerprint + "%"}
+	outfmt := []interface{}{
+		&cert.ID,
+		&cert.Fingerprint,
+		&cert.Type,
+		&cert.Name,
+		&cert.Certificate,
+	}
+
+	query := `
+		SELECT
+			id, fingerprint, type, name, certificate
+		FROM
+			certificates
+		WHERE fingerprint LIKE ?`
+
+	if err = dbQueryRowScan(db, query, inargs, outfmt); err != nil {
+		return nil, err
+	}
+
+	return cert, err
+}
+
+// dbCertSave stores a CertBaseInfo object in the db,
+// it will ignore the ID field from the dbCertInfo.
+func dbCertSave(db *sql.DB, cert *dbCertInfo) error {
+	tx, err := dbBegin(db)
+	if err != nil {
+		return err
+	}
+	stmt, err := tx.Prepare(`
+			INSERT INTO certificates (
+				fingerprint,
+				type,
+				name,
+				certificate
+			) VALUES (?, ?, ?, ?)`,
+	)
+	if err != nil {
+		tx.Rollback()
+		return err
+	}
+	defer stmt.Close()
+	_, err = stmt.Exec(
+		cert.Fingerprint,
+		cert.Type,
+		cert.Name,
+		cert.Certificate,
+	)
+	if err != nil {
+		tx.Rollback()
+		return err
+	}
+
+	return txCommit(tx)
+}
+
+// dbCertDelete deletes a certificate from the db.
+func dbCertDelete(db *sql.DB, fingerprint string) error {
+	_, err := dbExec(
+		db,
+		"DELETE FROM certificates WHERE fingerprint=?",
+		fingerprint,
+	)
+
+	return err
+}
+
+// dbImageGet gets an ImageBaseInfo object from the database.
+// The argument fingerprint will be queried with a LIKE query, means you can
+// pass a shortform and will get the full fingerprint.
 // There can never be more than one image with a given fingerprint, as it is
 // enforced by a UNIQUE constraint in the schema.
 func dbImageGet(db *sql.DB, fingerprint string, public bool) (*shared.ImageBaseInfo, error) {
@@ -717,7 +864,7 @@ func dbImageGet(db *sql.DB, fingerprint string, public bool) (*shared.ImageBaseI
 		query = query + " AND public=1"
 	}
 
-	err = shared.DbQueryRowScan(db, query, inargs, outfmt)
+	err = dbQueryRowScan(db, query, inargs, outfmt)
 
 	if err != nil {
 		return nil, err // Likely: there are no rows for this fingerprint
@@ -752,7 +899,7 @@ func dbAliasGet(db *sql.DB, name string) (fingerprint string, err error) {
 	inargs := []interface{}{name}
 	outfmt := []interface{}{&fingerprint}
 
-	err = shared.DbQueryRowScan(db, q, inargs, outfmt)
+	err = dbQueryRowScan(db, q, inargs, outfmt)
 
 	if err == sql.ErrNoRows {
 		return "", NoSuchObjectError
@@ -764,22 +911,22 @@ func dbAliasGet(db *sql.DB, name string) (fingerprint string, err error) {
 }
 
 // Insert an alias into the database.
-func dbAddAlias(db *sql.DB, name string, imageId int, desc string) error {
+func dbAddAlias(db *sql.DB, name string, imageID int, desc string) error {
 	stmt := `INSERT into images_aliases (name, image_id, description) values (?, ?, ?)`
-	_, err := shared.DbExec(db, stmt, name, imageId, desc)
+	_, err := dbExec(db, stmt, name, imageID, desc)
 	return err
 }
 
 // Get the container configuration map from the DB
-func dbGetConfig(db *sql.DB, containerId int) (map[string]string, error) {
+func dbGetConfig(db *sql.DB, containerID int) (map[string]string, error) {
 	var key, value string
 	q := `SELECT key, value FROM containers_config WHERE container_id=?`
 
-	inargs := []interface{}{containerId}
+	inargs := []interface{}{containerID}
 	outfmt := []interface{}{key, value}
 
 	// Results is already a slice here, not db Rows anymore.
-	results, err := shared.DbQueryScan(db, q, inargs, outfmt)
+	results, err := dbQueryScan(db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err //SmartError will wrap this and make "not found" errors pretty
 	}
@@ -807,7 +954,7 @@ func dbGetProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 		WHERE name=?`
 	inargs := []interface{}{name}
 	outfmt := []interface{}{key, value}
-	results, err := shared.DbQueryScan(db, query, inargs, outfmt)
+	results, err := dbQueryScan(db, query, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -819,7 +966,7 @@ func dbGetProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 		 */
 		query := "SELECT id FROM profiles WHERE name=?"
 		var id int
-		results, err := shared.DbQueryScan(db, query, []interface{}{name}, []interface{}{id})
+		results, err := dbQueryScan(db, query, []interface{}{name}, []interface{}{id})
 		if err != nil {
 			return nil, err
 		}
@@ -842,7 +989,7 @@ func dbGetProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 }
 
 // Get a list of profiles for a given container id.
-func dbGetProfiles(db *sql.DB, containerId int) ([]string, error) {
+func dbGetProfiles(db *sql.DB, containerID int) ([]string, error) {
 	var name string
 	var profiles []string
 
@@ -851,10 +998,10 @@ func dbGetProfiles(db *sql.DB, containerId int) ([]string, error) {
         JOIN profiles ON containers_profiles.profile_id=profiles.id
 		WHERE container_id=?
         ORDER BY containers_profiles.apply_order`
-	inargs := []interface{}{containerId}
+	inargs := []interface{}{containerID}
 	outfmt := []interface{}{name}
 
-	results, err := shared.DbQueryScan(db, query, inargs, outfmt)
+	results, err := dbQueryScan(db, query, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -881,7 +1028,7 @@ func dbGetDeviceConfig(db *sql.DB, id int, isprofile bool) (shared.Device, error
 		query = `SELECT key, value FROM containers_devices_config WHERE container_device_id=?`
 	}
 
-	results, err := shared.DbQueryScan(db, query, inargs, outfmt)
+	results, err := dbQueryScan(db, query, inargs, outfmt)
 
 	if err != nil {
 		return newdev, err
@@ -913,7 +1060,7 @@ func dbGetDevices(db *sql.DB, qName string, isprofile bool) (shared.Devices, err
 	var name, dtype string
 	inargs := []interface{}{qName}
 	outfmt := []interface{}{id, name, dtype}
-	results, err := shared.DbQueryScan(db, q, inargs, outfmt)
+	results, err := dbQueryScan(db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -939,15 +1086,181 @@ func dbListContainers(d *Daemon) ([]string, error) {
 	inargs := []interface{}{cTypeRegular}
 	var container string
 	outfmt := []interface{}{container}
-	result, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	result, err := dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
 
-	ret := make([]string, 0)
+	var ret []string
 	for _, container := range result {
 		ret = append(ret, container[0].(string))
 	}
 
 	return ret, nil
 }
+
+func isDbLockedError(err error) bool {
+	if err == nil {
+		return false
+	}
+	if err == sqlite3.ErrLocked || err == sqlite3.ErrBusy {
+		return true
+	}
+	if err.Error() == "database is locked" {
+		return true
+	}
+	return false
+}
+
+func dbBegin(db *sql.DB) (*sql.Tx, error) {
+	for {
+		tx, err := db.Begin()
+		if err == nil {
+			return tx, nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("DbBegin: error %q\n", err)
+			return nil, err
+		}
+		shared.Debugf("DbBegin: DB was locked\n")
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func txCommit(tx *sql.Tx) error {
+	for {
+		err := tx.Commit()
+		if err == nil {
+			return nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("Txcommit: error %q\n", err)
+			return err
+		}
+		shared.Debugf("Txcommit: db was locked\n")
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func dbQueryRowScan(db *sql.DB, q string, args []interface{}, outargs []interface{}) error {
+	for {
+		err := db.QueryRow(q, args...).Scan(outargs...)
+		if err == nil {
+			return nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("DbQuery: query %q error %q\n", q, err)
+			return err
+		}
+		shared.Debugf("DbQueryRowScan: query %q args %q, DB was locked\n", q, args)
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func dbQuery(db *sql.DB, q string, args ...interface{}) (*sql.Rows, error) {
+	for {
+		result, err := db.Query(q, args...)
+		if err == nil {
+			return result, nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("DbQuery: query %q error %q\n", q, err)
+			return nil, err
+		}
+		shared.Debugf("DbQuery: query %q args %q, DB was locked\n", q, args)
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface{}) ([][]interface{}, error) {
+	rows, err := db.Query(q, args...)
+	if err != nil {
+		return [][]interface{}{}, err
+	}
+	defer rows.Close()
+	result := [][]interface{}{}
+	for rows.Next() {
+		ptrargs := make([]interface{}, len(outargs))
+		for i := range outargs {
+			switch t := outargs[i].(type) {
+			case string:
+				str := ""
+				ptrargs[i] = &str
+			case int:
+				integer := 0
+				ptrargs[i] = &integer
+			default:
+				return [][]interface{}{}, fmt.Errorf("Bad interface type: %s\n", t)
+			}
+		}
+		err = rows.Scan(ptrargs...)
+		if err != nil {
+			return [][]interface{}{}, err
+		}
+		newargs := make([]interface{}, len(outargs))
+		for i := range ptrargs {
+			switch t := outargs[i].(type) {
+			case string:
+				newargs[i] = *ptrargs[i].(*string)
+			case int:
+				newargs[i] = *ptrargs[i].(*int)
+			default:
+				return [][]interface{}{}, fmt.Errorf("Bad interface type: %s\n", t)
+			}
+		}
+		result = append(result, newargs)
+	}
+	err = rows.Err()
+	if err != nil {
+		return [][]interface{}{}, err
+	}
+	return result, nil
+}
+
+/*
+ * . q is the database query
+ * . inargs is an array of interfaces containing the query arguments
+ * . outfmt is an array of interfaces containing the right types of output
+ *   arguments, i.e.
+ *      var arg1 string
+ *      var arg2 int
+ *      outfmt := {}interface{}{arg1, arg2}
+ *
+ * The result will be an array (one per output row) of arrays (one per output argument)
+ * of interfaces, containing pointers to the actual output arguments.
+ */
+func dbQueryScan(db *sql.DB, q string, inargs []interface{}, outfmt []interface{}) ([][]interface{}, error) {
+	for {
+		result, err := doDbQueryScan(db, q, inargs, outfmt)
+		if err == nil {
+			return result, nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("DbQuery: query %q error %q\n", q, err)
+			return nil, err
+		}
+		shared.Debugf("DbQueryscan: query %q inargs %q, DB was locked\n", q, inargs)
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func dbExec(db *sql.DB, q string, args ...interface{}) (sql.Result, error) {
+	for {
+		result, err := db.Exec(q, args...)
+		if err == nil {
+			return result, nil
+		}
+		if !isDbLockedError(err) {
+			shared.Debugf("DbExec: query %q error %q\n", q, err)
+			return nil, err
+		}
+		shared.Debugf("DbExec: query %q args %q, DB was locked\n", q, args)
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
+	}
+}
