@@ -7,10 +7,13 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
+	"mime"
+	"mime/multipart"
 	"net/http"
 	"net/url"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"strconv"
 	"strings"
 
@@ -20,14 +23,6 @@ import (
 	"github.com/lxc/lxd/shared"
 )
 
-const (
-	COMPRESSION_TAR = iota
-	COMPRESSION_GZIP
-	COMPRESSION_BZ2
-	COMPRESSION_LZMA
-	COMPRESSION_XZ
-)
-
 func getSize(f *os.File) (int64, error) {
 	fi, err := f.Stat()
 	if err != nil {
@@ -36,11 +31,10 @@ func getSize(f *os.File) (int64, error) {
 	return fi.Size(), nil
 }
 
-func detectCompression(fname string) (int, string, error) {
-
+func detectCompression(fname string) ([]string, string, error) {
 	f, err := os.Open(fname)
 	if err != nil {
-		return -1, "", err
+		return []string{""}, "", err
 	}
 	defer f.Close()
 
@@ -52,148 +46,40 @@ func detectCompression(fname string) (int, string, error) {
 	// tar - 263 bytes, trying to get ustar from 257 - 262
 	header := make([]byte, 263)
 	_, err = f.Read(header)
+	if err != nil {
+		return []string{""}, "", err
+	}
 
 	switch {
 	case bytes.Equal(header[0:2], []byte{'B', 'Z'}):
-		return COMPRESSION_BZ2, ".tar.bz2", nil
+		return []string{"--jxf"}, ".tar.bz2", nil
 	case bytes.Equal(header[0:2], []byte{0x1f, 0x8b}):
-		return COMPRESSION_GZIP, ".tar.gz", nil
+		return []string{"-zxf"}, ".tar.gz", nil
 	case (bytes.Equal(header[1:5], []byte{'7', 'z', 'X', 'Z'}) && header[0] == 0xFD):
-		return COMPRESSION_XZ, ".tar.xz", nil
+		return []string{"-Jxf"}, ".tar.xz", nil
 	case (bytes.Equal(header[1:5], []byte{'7', 'z', 'X', 'Z'}) && header[0] != 0xFD):
-		return COMPRESSION_LZMA, ".tar.lzma", nil
+		return []string{"--lzma", "-xf"}, ".tar.lzma", nil
 	case bytes.Equal(header[257:262], []byte{'u', 's', 't', 'a', 'r'}):
-		return COMPRESSION_TAR, ".tar", nil
+		return []string{"-xf"}, ".tar", nil
 	default:
-		return -1, "", fmt.Errorf("Unsupported compression.")
+		return []string{""}, "", fmt.Errorf("Unsupported compression.")
 	}
 
 }
 
-type imageFromContainerPostReq struct {
-	Name       string            `json:"name"`
-	Public     bool              `json:"public"`
-	Source     map[string]string `json:"source"`
-	Properties map[string]string `json:"properties"`
-}
-
-type imageMetadata struct {
-	Architecture  string
-	Creation_date float64
-	Properties    map[string]interface{}
-	Templates     map[string]*TemplateEntry
-}
-
-func imgPostContInfo(r *http.Request, req imageFromContainerPostReq, builddir string) (public int,
-	fingerprint string, arch int,
-	filename string, size int64,
-	properties map[string]string, err error) {
-	return 0, "", 0, "", 0, properties, fmt.Errorf("Not implemented")
-}
-
-func getImgPostInfo(r *http.Request, builddir string) (public int,
-	fingerprint string, arch int,
-	filename string, size int64,
-	properties map[string]string, err error) {
-
-	// Is this a container request?
-	decoder := json.NewDecoder(r.Body)
-	req := imageFromContainerPostReq{}
-	if err = decoder.Decode(&req); err == nil {
-		return imgPostContInfo(r, req, builddir)
-	}
-
-	// ok we've got an image in the body
-	public, _ = strconv.Atoi(r.Header.Get("X-LXD-public"))
-	filename = r.Header.Get("X-LXD-filename")
-	propHeaders := r.Header[http.CanonicalHeaderKey("X-LXD-properties")]
-
-	properties = map[string]string{}
-	if len(propHeaders) > 0 {
-		for _, ph := range propHeaders {
-			p, _ := url.ParseQuery(ph)
-			for pkey, pval := range p {
-				properties[pkey] = pval[0]
-			}
-		}
-	}
-
-	// Create a file for the tarball
-	tarf, err := ioutil.TempFile(builddir, "lxd_tar_")
-	if err != nil {
-		return 0, "", 0, "", 0, properties, err
-	}
-
-	tarfname := tarf.Name()
-	sha256 := sha256.New()
-
-	var size1, size2 int64
-	size1, err = io.Copy(io.MultiWriter(tarf, sha256), decoder.Buffered())
-	if err == nil {
-		size2, err = io.Copy(io.MultiWriter(tarf, sha256), r.Body)
-	}
-	size = size1 + size2
-	tarf.Close()
-	if err != nil {
-		return 0, "", 0, "", 0, properties, err
-	}
-
-	fingerprint = fmt.Sprintf("%x", sha256.Sum(nil))
-	expectedFingerprint := r.Header.Get("X-LXD-fingerprint")
-	if expectedFingerprint != "" && fingerprint != expectedFingerprint {
-		err = fmt.Errorf("fingerprints don't match, got %s expected %s", fingerprint, expectedFingerprint)
-		return 0, "", 0, "", 0, properties, err
-	}
-
-	imagefname := fmt.Sprintf("%s/%s", builddir, fingerprint)
-	err = os.Rename(tarfname, imagefname)
-	if err != nil {
-		return 0, "", 0, "", 0, properties, err
-	}
-
-	var imageMeta *imageMetadata
-	imageMeta, err = getImageMetadata(imagefname)
-	if err != nil {
-		return 0, "", 0, "", 0, properties, err
-	}
-
-	arch, _ = shared.ArchitectureId(imageMeta.Architecture)
-
-	err = nil
-	return
-}
-
-func makeBtrfsSubvol(imagefname, subvol string) error {
-	output, err := exec.Command("btrfs", "subvolume", "create", subvol).CombinedOutput()
-	if err != nil {
-		shared.Debugf("btrfs subvolume creation failed\n")
-		shared.Debugf(string(output))
-		return err
-	}
-
-	compression, _, err := detectCompression(imagefname)
+func untar(tarball string, path string) error {
+	extractArgs, _, err := detectCompression(tarball)
 	if err != nil {
 		return err
 	}
 
-	args := []string{"-C", subvol, "--numeric-owner"}
-	switch compression {
-	case COMPRESSION_TAR:
-		args = append(args, "-xf")
-	case COMPRESSION_GZIP:
-		args = append(args, "-zxf")
-	case COMPRESSION_BZ2:
-		args = append(args, "-jxf")
-	case COMPRESSION_LZMA:
-		args = append(args, "--lzma", "-xf")
-	default:
-		args = append(args, "-Jxf")
-	}
-	args = append(args, imagefname)
+	args := []string{"-C", path, "--numeric-owner"}
+	args = append(args, extractArgs...)
+	args = append(args, tarball)
 
-	output, err = exec.Command("tar", args...).CombinedOutput()
+	output, err := exec.Command("tar", args...).CombinedOutput()
 	if err != nil {
-		shared.Debugf("image unpacking failed\n")
+		shared.Debugf("unpacking failed\n")
 		shared.Debugf(string(output))
 		return err
 	}
@@ -201,14 +87,345 @@ func makeBtrfsSubvol(imagefname, subvol string) error {
 	return nil
 }
 
+func untarImage(imagefname string, destpath string) error {
+	err := untar(imagefname, destpath)
+	if err != nil {
+		return err
+	}
+
+	if shared.PathExists(imagefname + ".rootfs") {
+		rootfsPath := fmt.Sprintf("%s/rootfs", destpath)
+		err = os.MkdirAll(rootfsPath, 0700)
+		if err != nil {
+			return fmt.Errorf("Error creating rootfs directory")
+		}
+
+		err = untar(imagefname+".rootfs", rootfsPath)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+type imagePostReq struct {
+	Filename   string            `json:"filename"`
+	Public     bool              `json:"public"`
+	Source     map[string]string `json:"source"`
+	Properties map[string]string `json:"properties"`
+}
+
+type imageMetadata struct {
+	Architecture string                    `yaml:"architecture"`
+	CreationDate int64                     `yaml:"creation_date"`
+	ExpiryDate   int64                     `yaml:"expiry_date"`
+	Properties   map[string]string         `yaml:"properties"`
+	Templates    map[string]*TemplateEntry `yaml:"templates"`
+}
+
+/*
+ * This function takes a container or snapshot from the local image server and
+ * exports it as an image.
+ */
+func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq, builddir string) (info shared.ImageInfo, err error) {
+
+	info.Properties = map[string]string{}
+	name := req.Source["name"]
+	ctype := req.Source["type"]
+	if ctype == "" || name == "" {
+		return info, fmt.Errorf("No source provided")
+	}
+
+	switch ctype {
+	case "snapshot":
+		if !shared.IsSnapshot(name) {
+			return info, fmt.Errorf("Not a snapshot")
+		}
+	case "container":
+		if shared.IsSnapshot(name) {
+			return info, fmt.Errorf("This is a snapshot")
+		}
+	default:
+		return info, fmt.Errorf("Bad type")
+	}
+
+	info.Filename = req.Filename
+	switch req.Public {
+	case true:
+		info.Public = 1
+	case false:
+		info.Public = 0
+	}
+
+	snap := ""
+	if ctype == "snapshot" {
+		fields := strings.SplitN(name, "/", 2)
+		if len(fields) != 2 {
+			return info, fmt.Errorf("Not a snapshot")
+		}
+		name = fields[0]
+		snap = fields[1]
+	}
+
+	c, err := newLxdContainer(name, d)
+	if err != nil {
+		return info, err
+	}
+
+	// Build the actual image file
+	tarfname := fmt.Sprintf("%s.tar", name)
+	tarpath := filepath.Join(builddir, tarfname)
+	tarfile, err := os.OpenFile(tarpath, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
+	if err != nil {
+		return info, err
+	}
+	if err := c.exportToTar(snap, tarfile); err != nil {
+		tarfile.Close()
+		return info, fmt.Errorf("imgPostContInfo: exportToTar failed: %s\n", err)
+	}
+	tarfile.Close()
+
+	args := []string{tarpath}
+	_, err = exec.Command("gzip", args...).CombinedOutput()
+	if err != nil {
+		shared.Debugf("image compression\n")
+		return info, err
+	}
+	gztarpath := fmt.Sprintf("%s.gz", tarpath)
+
+	sha256 := sha256.New()
+	tarf, err := os.Open(gztarpath)
+	if err != nil {
+		return info, err
+	}
+	info.Size, err = io.Copy(sha256, tarf)
+	tarf.Close()
+	if err != nil {
+		return info, err
+	}
+	info.Fingerprint = fmt.Sprintf("%x", sha256.Sum(nil))
+
+	/* rename the the file to the expected name so our caller can use it */
+	imagefname := filepath.Join(builddir, info.Fingerprint)
+	err = os.Rename(gztarpath, imagefname)
+	if err != nil {
+		return info, err
+	}
+
+	info.Architecture = c.architecture
+	info.Properties = req.Properties
+
+	return info, nil
+}
+
+func imgPostRemoteInfo(d *Daemon, req imagePostReq) Response {
+	var err error
+	var hash string
+
+	if req.Source["alias"] != "" {
+		if req.Source["mode"] == "pull" && req.Source["server"] != "" {
+			hash, err = remoteGetImageFingerprint(d, req.Source["server"], req.Source["alias"])
+			if err != nil {
+				return InternalError(err)
+			}
+		} else {
+
+			hash, err = dbAliasGet(d.db, req.Source["alias"])
+			if err != nil {
+				return InternalError(err)
+			}
+		}
+	} else if req.Source["fingerprint"] != "" {
+		hash = req.Source["fingerprint"]
+	} else {
+		return BadRequest(fmt.Errorf("must specify one of alias or fingerprint for init from image"))
+	}
+
+	err = ensureLocalImage(d, req.Source["server"], hash, req.Source["secret"])
+	if err != nil {
+		return InternalError(err)
+	}
+
+	info, err := dbImageGet(d.db, hash, false)
+	if err != nil {
+		return InternalError(err)
+	}
+
+	metadata := make(map[string]string)
+	metadata["fingerprint"] = info.Fingerprint
+	metadata["size"] = strconv.FormatInt(info.Size, 10)
+
+	return SyncResponse(true, metadata)
+}
+
+func getImgPostInfo(d *Daemon, r *http.Request, builddir string, post *os.File) (info shared.ImageInfo, err error) {
+	var imageMeta *imageMetadata
+
+	info.Public, _ = strconv.Atoi(r.Header.Get("X-LXD-public"))
+	propHeaders := r.Header[http.CanonicalHeaderKey("X-LXD-properties")]
+	ctype, ctypeParams, err := mime.ParseMediaType(r.Header.Get("Content-Type"))
+	if err != nil {
+		ctype = "application/octet-stream"
+	}
+
+	sha256 := sha256.New()
+	var size int64
+
+	// Create a temporary file for the image tarball
+	imageTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
+	if err != nil {
+		return info, err
+	}
+
+	if ctype == "multipart/form-data" {
+		// Create a temporary file for the rootfs tarball
+		rootfsTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
+		if err != nil {
+			return info, err
+		}
+
+		// Parse the POST data
+		post.Seek(0, 0)
+		mr := multipart.NewReader(post, ctypeParams["boundary"])
+
+		// Get the metadata tarball
+		part, err := mr.NextPart()
+		if err != nil {
+			return info, err
+		}
+
+		if part.FormName() != "metadata" {
+			return info, fmt.Errorf("Invalid multipart image")
+		}
+
+		size, err = io.Copy(io.MultiWriter(imageTarf, sha256), part)
+		info.Size += size
+
+		imageTarf.Close()
+		if err != nil {
+			return info, err
+		}
+
+		// Get the rootfs tarball
+		part, err = mr.NextPart()
+		if err != nil {
+			return info, err
+		}
+
+		if part.FormName() != "rootfs" {
+			return info, fmt.Errorf("Invalid multipart image")
+		}
+
+		size, err = io.Copy(io.MultiWriter(rootfsTarf, sha256), part)
+		info.Size += size
+
+		rootfsTarf.Close()
+		if err != nil {
+			return info, err
+		}
+
+		info.Filename = part.FileName()
+		info.Fingerprint = fmt.Sprintf("%x", sha256.Sum(nil))
+
+		expectedFingerprint := r.Header.Get("X-LXD-fingerprint")
+		if expectedFingerprint != "" && info.Fingerprint != expectedFingerprint {
+			err = fmt.Errorf("fingerprints don't match, got %s expected %s", info.Fingerprint, expectedFingerprint)
+			return info, err
+		}
+
+		imgfname := filepath.Join(builddir, info.Fingerprint)
+		err = os.Rename(imageTarf.Name(), imgfname)
+		if err != nil {
+			return info, err
+		}
+
+		rootfsfname := filepath.Join(builddir, info.Fingerprint+".rootfs")
+		err = os.Rename(rootfsTarf.Name(), rootfsfname)
+		if err != nil {
+			return info, err
+		}
+
+		imageMeta, err = getImageMetadata(imgfname)
+		if err != nil {
+			return info, err
+		}
+	} else {
+		post.Seek(0, 0)
+		size, err = io.Copy(io.MultiWriter(imageTarf, sha256), post)
+		info.Size = size
+		imageTarf.Close()
+		if err != nil {
+			return info, err
+		}
+
+		info.Filename = r.Header.Get("X-LXD-filename")
+		info.Fingerprint = fmt.Sprintf("%x", sha256.Sum(nil))
+
+		expectedFingerprint := r.Header.Get("X-LXD-fingerprint")
+		if expectedFingerprint != "" && info.Fingerprint != expectedFingerprint {
+			err = fmt.Errorf("fingerprints don't match, got %s expected %s", info.Fingerprint, expectedFingerprint)
+			return info, err
+		}
+
+		imgfname := filepath.Join(builddir, info.Fingerprint)
+		err = os.Rename(imageTarf.Name(), imgfname)
+		if err != nil {
+			return info, err
+		}
+
+		imageMeta, err = getImageMetadata(imgfname)
+		if err != nil {
+			return info, err
+		}
+	}
+
+	info.Architecture, _ = shared.ArchitectureId(imageMeta.Architecture)
+	info.CreationDate = imageMeta.CreationDate
+	info.ExpiryDate = imageMeta.ExpiryDate
+
+	info.Properties = imageMeta.Properties
+	if len(propHeaders) > 0 {
+		for _, ph := range propHeaders {
+			p, _ := url.ParseQuery(ph)
+			for pkey, pval := range p {
+				info.Properties[pkey] = pval[0]
+			}
+		}
+	}
+
+	return info, nil
+}
+
 func removeImgWorkdir(d *Daemon, builddir string) {
+	vgname, _, err := getServerConfigValue(d, "core.lvm_vg_name")
+	if err != nil {
+		shared.Debugf("Error checking server config: %v", err)
+	}
+
+	matches, _ := filepath.Glob(fmt.Sprintf("%s/*.lv", builddir))
+	if len(matches) > 0 {
+		if len(matches) > 1 {
+			shared.Debugf("Unexpected - more than one .lv file in builddir. using first: %v", matches)
+		}
+		lvsymlink := matches[0]
+		if lvpath, err := os.Readlink(lvsymlink); err != nil {
+			shared.Debugf("Error reading target of symlink '%s'", lvsymlink)
+		} else {
+			err = shared.LVMRemoveLV(vgname, filepath.Base(lvpath))
+			if err != nil {
+				shared.Debugf("Error removing LV '%s': %v", lvpath, err)
+			}
+		}
+	}
+
 	if d.BackingFs == "btrfs" {
 		/* cannot rm -rf /a if /a/b is a subvolume, so first delete subvolumes */
 		/* todo: find the .btrfs file under dir */
 		fnamelist, _ := shared.ReadDir(builddir)
 		for _, fname := range fnamelist {
-			subvol := fmt.Sprintf("%s/%s", builddir, fname)
-			exec.Command("btrfs", "subvolume", "delete", subvol).Run()
+			subvol := filepath.Join(builddir, fname)
+			btrfsDeleteSubvol(subvol)
 		}
 	}
 	if remErr := os.RemoveAll(builddir); remErr != nil {
@@ -216,33 +433,125 @@ func removeImgWorkdir(d *Daemon, builddir string) {
 	}
 }
 
-// We've got an image with the directory, create .btrfs or .lvm
+// We've got an image with the directory, create .btrfs or .lv
 func buildOtherFs(d *Daemon, builddir string, fp string) error {
+	vgname, vgnameIsSet, err := getServerConfigValue(d, "core.lvm_vg_name")
+	if err != nil {
+		return fmt.Errorf("Error checking server config: %v", err)
+	}
+
+	if vgnameIsSet {
+		return createImageLV(d, builddir, fp, vgname)
+	}
+
 	switch d.BackingFs {
 	case "btrfs":
-		imagefname := fmt.Sprintf("%s/%s", builddir, fp)
+		imagefname := filepath.Join(builddir, fp)
 		subvol := fmt.Sprintf("%s.btrfs", imagefname)
-		if err := makeBtrfsSubvol(imagefname, subvol); err != nil {
+		if err := btrfsMakeSubvol(subvol); err != nil {
+			return err
+		}
+
+		err = untarImage(imagefname, subvol)
+		if err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
+func createImageLV(d *Daemon, builddir string, fingerprint string, vgname string) error {
+	imagefname := filepath.Join(builddir, fingerprint)
+	poolname, poolnameIsSet, err := getServerConfigValue(d, "core.lvm_thinpool_name")
+	if err != nil {
+		return fmt.Errorf("Error checking server config: %v", err)
+	}
+
+	if !poolnameIsSet {
+		poolname, err = shared.LVMCreateDefaultThinPool(vgname)
+		if err != nil {
+			return fmt.Errorf("Error creating LVM thin pool: %v", err)
+		}
+		err = setLVMThinPoolNameConfig(d, poolname)
+		if err != nil {
+			shared.Debugf("Error setting thin pool name: '%s'", err)
+			return fmt.Errorf("Error setting LVM thin pool config: %v", err)
+		}
+	}
+
+	lvpath, err := shared.LVMCreateThinLV(fingerprint, poolname, vgname)
+	if err != nil {
+		shared.Logf("Error from LVMCreateThinLV: '%v'", err)
+		return fmt.Errorf("Error Creating LVM LV for new image: %v", err)
+	}
+
+	err = os.Symlink(lvpath, fmt.Sprintf("%s.lv", imagefname))
+	if err != nil {
+		return err
+	}
+
+	output, err := exec.Command("mkfs.ext4", "-E", "nodiscard,lazy_itable_init=0,lazy_journal_init=0", lvpath).CombinedOutput()
+	if err != nil {
+		shared.Logf("Error output from mkfs.ext4: '%s'", output)
+		return fmt.Errorf("Error making filesystem on image LV: %v", err)
+	}
+
+	tempLVMountPoint, err := ioutil.TempDir(builddir, "tmp_lv_mnt")
+	if err != nil {
+		return err
+	}
+
+	output, err = exec.Command("mount", "-o", "discard", lvpath, tempLVMountPoint).CombinedOutput()
+	if err != nil {
+		shared.Logf("Error mounting image LV for untarring: '%s'", output)
+		return fmt.Errorf("Error mounting image LV: %v", err)
+
+	}
+
+	untarErr := untarImage(imagefname, tempLVMountPoint)
+
+	output, err = exec.Command("umount", tempLVMountPoint).CombinedOutput()
+	if err != nil {
+		shared.Logf("WARNING: could not unmount LV '%s' from '%s'. Will not remove. Error: %v", lvpath, tempLVMountPoint, err)
+		if untarErr == nil {
+			return err
+		}
+
+		return fmt.Errorf("Error unmounting '%s' during cleanup of error %v", tempLVMountPoint, untarErr)
+	}
+
+	return untarErr
+}
+
 // Copy imagefile and btrfs file out of the tmpdir
-func pullOutImagefiles(d *Daemon, builddir string, fp string) error {
-	imagefname := fmt.Sprintf("%s/%s", builddir, fp)
-	finalName := shared.VarPath("images", fp)
-	subvol := fmt.Sprintf("%s.btrfs", imagefname)
+func pullOutImagefiles(d *Daemon, builddir string, fingerprint string) error {
+	imagefname := filepath.Join(builddir, fingerprint)
+	imagerootfsfname := filepath.Join(builddir, fingerprint+".rootfs")
+	finalName := shared.VarPath("images", fingerprint)
+	finalrootfsName := shared.VarPath("images", fingerprint+".rootfs")
+
+	if shared.PathExists(imagerootfsfname) {
+		err := os.Rename(imagerootfsfname, finalrootfsName)
+		if err != nil {
+			return err
+		}
+	}
 
 	err := os.Rename(imagefname, finalName)
 	if err != nil {
 		return err
 	}
 
+	lvsymlink := fmt.Sprintf("%s.lv", imagefname)
+	if shared.PathExists(lvsymlink) {
+		dst := shared.VarPath("images", fmt.Sprintf("%s.lv", fingerprint))
+		return os.Rename(lvsymlink, dst)
+	}
+
 	switch d.BackingFs {
 	case "btrfs":
-		dst := shared.VarPath("images", fmt.Sprintf("%s.btrfs", fp))
+		subvol := fmt.Sprintf("%s.btrfs", imagefname)
+		dst := shared.VarPath("images", fmt.Sprintf("%s.btrfs", fingerprint))
 		if err := os.Rename(subvol, dst); err != nil {
 			return err
 		}
@@ -252,20 +561,20 @@ func pullOutImagefiles(d *Daemon, builddir string, fp string) error {
 }
 
 func dbInsertImage(d *Daemon, fp string, fname string, sz int64, public int,
-	arch int, properties map[string]string) error {
-	tx, err := shared.DbBegin(d.db)
+	arch int, creationDate int64, expiryDate int64, properties map[string]string) error {
+	tx, err := dbBegin(d.db)
 	if err != nil {
 		return err
 	}
 
-	stmt, err := tx.Prepare(`INSERT INTO images (fingerprint, filename, size, public, architecture, upload_date) VALUES (?, ?, ?, ?, ?, strftime("%s"))`)
+	stmt, err := tx.Prepare(`INSERT INTO images (fingerprint, filename, size, public, architecture, creation_date, expiry_date, upload_date) VALUES (?, ?, ?, ?, ?, ?, ?, strftime("%s"))`)
 	if err != nil {
 		tx.Rollback()
 		return err
 	}
 	defer stmt.Close()
 
-	result, err := stmt.Exec(fp, fname, sz, public, arch)
+	result, err := stmt.Exec(fp, fname, sz, public, arch, creationDate, expiryDate)
 	if err != nil {
 		tx.Rollback()
 		return err
@@ -300,7 +609,7 @@ func dbInsertImage(d *Daemon, fp string, fname string, sz int64, public int,
 
 	}
 
-	if err := shared.TxCommit(tx); err != nil {
+	if err := txCommit(tx); err != nil {
 		return err
 	}
 
@@ -308,6 +617,8 @@ func dbInsertImage(d *Daemon, fp string, fname string, sz int64, public int,
 }
 
 func imagesPost(d *Daemon, r *http.Request) Response {
+	var err error
+	var info shared.ImageInfo
 
 	dirname := shared.VarPath("images")
 	if err := os.MkdirAll(dirname, 0700); err != nil {
@@ -323,33 +634,75 @@ func imagesPost(d *Daemon, r *http.Request) Response {
 	/* remove the builddir when done */
 	defer removeImgWorkdir(d, builddir)
 
-	/* Grab all info from the web request */
-	public, fingerprint, arch, filename, size, properties, err := getImgPostInfo(r, builddir)
+	// Store the post data to disk
+	post, err := ioutil.TempFile(builddir, "lxd_post_")
+	if err != nil {
+		return InternalError(err)
+	}
+
+	os.Remove(post.Name())
+	defer post.Close()
+
+	_, err = io.Copy(post, r.Body)
+	if err != nil {
+		return InternalError(err)
+	}
+
+	// Is this a container request?
+	post.Seek(0, 0)
+	decoder := json.NewDecoder(post)
+	req := imagePostReq{}
+	err = decoder.Decode(&req)
+
+	if err == nil {
+		/* Processing image request */
+		if req.Source["type"] == "container" || req.Source["type"] == "snapshot" {
+			info, err = imgPostContInfo(d, r, req, builddir)
+			if err != nil {
+				return SmartError(err)
+			}
+		} else if req.Source["type"] == "image" {
+			return imgPostRemoteInfo(d, req)
+		} else {
+			return InternalError(fmt.Errorf("Invalid images JSON"))
+		}
+	} else {
+		/* Processing image upload */
+		info, err = getImgPostInfo(d, r, builddir, post)
+		if err != nil {
+			return SmartError(err)
+		}
+	}
+
+	metadata, err := buildImageFromInfo(d, info, builddir)
 	if err != nil {
 		return SmartError(err)
 	}
+	return SyncResponse(true, metadata)
+}
 
-	if err := buildOtherFs(d, builddir, fingerprint); err != nil {
-		return SmartError(err)
+func buildImageFromInfo(d *Daemon, info shared.ImageInfo, builddir string) (metadata map[string]string, err error) {
+	if err := buildOtherFs(d, builddir, info.Fingerprint); err != nil {
+		return nil, err
 	}
 
-	err = dbInsertImage(d, fingerprint, filename, size, public, arch, properties)
+	err = dbInsertImage(d, info.Fingerprint, info.Filename, info.Size, info.Public, info.Architecture, info.CreationDate, info.ExpiryDate, info.Properties)
 	if err != nil {
-		return SmartError(err)
+		return nil, err
 	}
 
-	metadata := make(map[string]string)
-	metadata["fingerprint"] = fingerprint
-	metadata["size"] = strconv.FormatInt(size, 10)
+	metadata = make(map[string]string)
+	metadata["fingerprint"] = info.Fingerprint
+	metadata["size"] = strconv.FormatInt(info.Size, 10)
 
-	err = pullOutImagefiles(d, builddir, fingerprint)
+	err = pullOutImagefiles(d, builddir, info.Fingerprint)
 	if err != nil {
-		return SmartError(err)
+		return nil, err
 	}
 
 	// now we can let the deferred cleanup fn remove the tmpdir
 
-	return SyncResponse(true, metadata)
+	return metadata, nil
 }
 
 func xzReader(r io.Reader) io.ReadCloser {
@@ -368,28 +721,16 @@ func xzReader(r io.Reader) io.ReadCloser {
 }
 
 func getImageMetadata(fname string) (*imageMetadata, error) {
-
 	metadataName := "metadata.yaml"
 
-	compression, _, err := detectCompression(fname)
+	compressionArgs, _, err := detectCompression(fname)
 
 	if err != nil {
 		return nil, err
 	}
 
 	args := []string{"-O"}
-	switch compression {
-	case COMPRESSION_TAR:
-		args = append(args, "-xf")
-	case COMPRESSION_GZIP:
-		args = append(args, "-zxf")
-	case COMPRESSION_BZ2:
-		args = append(args, "--jxf")
-	case COMPRESSION_LZMA:
-		args = append(args, "--lzma", "-xf")
-	default:
-		args = append(args, "-Jxf")
-	}
+	args = append(args, compressionArgs...)
 	args = append(args, fname, metadataName)
 
 	shared.Debugf("Extracting tarball using command: tar %s", strings.Join(args, " "))
@@ -423,8 +764,8 @@ func imagesGet(d *Daemon, r *http.Request) Response {
 }
 
 func doImagesGet(d *Daemon, recursion bool, public bool) (interface{}, error) {
-	result_string := make([]string, 0)
-	result_map := make([]shared.ImageInfo, 0)
+	resultString := []string{}
+	resultMap := []shared.ImageInfo{}
 
 	q := "SELECT fingerprint FROM images"
 	var name string
@@ -433,7 +774,7 @@ func doImagesGet(d *Daemon, recursion bool, public bool) (interface{}, error) {
 	}
 	inargs := []interface{}{}
 	outfmt := []interface{}{name}
-	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	results, err := dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return []string{}, err
 	}
@@ -442,31 +783,26 @@ func doImagesGet(d *Daemon, recursion bool, public bool) (interface{}, error) {
 		name = r[0].(string)
 		if !recursion {
 			url := fmt.Sprintf("/%s/images/%s", shared.APIVersion, name)
-			result_string = append(result_string, url)
+			resultString = append(resultString, url)
 		} else {
 			image, response := doImageGet(d, name, public)
 			if response != nil {
 				continue
 			}
-			result_map = append(result_map, image)
+			resultMap = append(resultMap, image)
 		}
 	}
 
 	if !recursion {
-		return result_string, nil
-	} else {
-		return result_map, nil
+		return resultString, nil
 	}
+
+	return resultMap, nil
 }
 
-var imagesCmd = Command{name: "images", post: imagesPost, get: imagesGet}
+var imagesCmd = Command{name: "images", post: imagesPost, untrustedGet: true, get: imagesGet}
 
 func imageDelete(d *Daemon, r *http.Request) Response {
-	backing_fs, err := shared.GetFilesystem(d.lxcpath)
-	if err != nil {
-		return InternalError(err)
-	}
-
 	fingerprint := mux.Vars(r)["fingerprint"]
 
 	imgInfo, err := dbImageGet(d.db, fingerprint, false)
@@ -480,12 +816,36 @@ func imageDelete(d *Daemon, r *http.Request) Response {
 		shared.Debugf("Error deleting image file %s: %s\n", fname, err)
 	}
 
-	if backing_fs == "btrfs" {
-		subvol := fmt.Sprintf("%s.btrfs", fname)
-		exec.Command("btrfs", "subvolume", "delete", subvol).Run()
+	fmetaname := shared.VarPath("images", imgInfo.Fingerprint+".rootfs")
+	if shared.PathExists(fmetaname) {
+		err = os.Remove(fmetaname)
+		if err != nil {
+			shared.Debugf("Error deleting image file %s: %s\n", fmetaname, err)
+		}
 	}
 
-	tx, err := shared.DbBegin(d.db)
+	vgname, vgnameIsSet, err := getServerConfigValue(d, "core.lvm_vg_name")
+	if err != nil {
+		return InternalError(fmt.Errorf("Error checking server config: %v", err))
+	}
+
+	if vgnameIsSet {
+		err = shared.LVMRemoveLV(vgname, imgInfo.Fingerprint)
+		if err != nil {
+			return InternalError(fmt.Errorf("Failed to remove deleted image LV: %v", err))
+		}
+
+		lvsymlink := fmt.Sprintf("%s.lv", fname)
+		err = os.Remove(lvsymlink)
+		if err != nil {
+			return InternalError(fmt.Errorf("Failed to remove symlink to deleted image LV: '%s': %v", lvsymlink, err))
+		}
+	} else if d.BackingFs == "btrfs" {
+		subvol := fmt.Sprintf("%s.btrfs", fname)
+		btrfsDeleteSubvol(subvol)
+	}
+
+	tx, err := dbBegin(d.db)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -494,7 +854,7 @@ func imageDelete(d *Daemon, r *http.Request) Response {
 	_, _ = tx.Exec("DELETE FROM images_properties WHERE image_id?", imgInfo.Id)
 	_, _ = tx.Exec("DELETE FROM images WHERE id=?", imgInfo.Id)
 
-	if err := shared.TxCommit(tx); err != nil {
+	if err := txCommit(tx); err != nil {
 		return InternalError(err)
 	}
 
@@ -511,7 +871,7 @@ func doImageGet(d *Daemon, fingerprint string, public bool) (shared.ImageInfo, R
 	var key, value, name, desc string
 	inargs := []interface{}{imgInfo.Id}
 	outfmt := []interface{}{key, value}
-	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	results, err := dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return shared.ImageInfo{}, SmartError(err)
 	}
@@ -525,7 +885,7 @@ func doImageGet(d *Daemon, fingerprint string, public bool) (shared.ImageInfo, R
 	q = "SELECT name, description FROM images_aliases WHERE image_id=?"
 	inargs = []interface{}{imgInfo.Id}
 	outfmt = []interface{}{name, desc}
-	results, err = shared.DbQueryScan(d.db, q, inargs, outfmt)
+	results, err = dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return shared.ImageInfo{}, InternalError(err)
 	}
@@ -637,7 +997,7 @@ func imagePut(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	tx, err := shared.DbBegin(d.db)
+	tx, err := dbBegin(d.db)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -658,7 +1018,7 @@ func imagePut(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	if err := shared.TxCommit(tx); err != nil {
+	if err := txCommit(tx); err != nil {
 		return InternalError(err)
 	}
 
@@ -712,32 +1072,32 @@ func aliasesGet(d *Daemon, r *http.Request) Response {
 	var name string
 	inargs := []interface{}{}
 	outfmt := []interface{}{name}
-	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	results, err := dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return BadRequest(err)
 	}
-	response_str := make([]string, 0)
-	response_map := make([]shared.ImageAlias, 0)
+	responseStr := []string{}
+	responseMap := []shared.ImageAlias{}
 	for _, res := range results {
 		name = res[0].(string)
 		if !recursion {
 			url := fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, name)
-			response_str = append(response_str, url)
+			responseStr = append(responseStr, url)
 
 		} else {
 			alias, err := doAliasGet(d, name, d.isTrustedClient(r))
 			if err != nil {
 				continue
 			}
-			response_map = append(response_map, alias)
+			responseMap = append(responseMap, alias)
 		}
 	}
 
 	if !recursion {
-		return SyncResponse(true, response_str)
-	} else {
-		return SyncResponse(true, response_map)
+		return SyncResponse(true, responseStr)
 	}
+
+	return SyncResponse(true, responseMap)
 }
 
 func aliasGet(d *Daemon, r *http.Request) Response {
@@ -764,7 +1124,7 @@ func doAliasGet(d *Daemon, name string, isTrustedClient bool) (shared.ImageAlias
 	var fingerprint, description string
 	arg1 := []interface{}{name}
 	arg2 := []interface{}{&fingerprint, &description}
-	err := shared.DbQueryRowScan(d.db, q, arg1, arg2)
+	err := dbQueryRowScan(d.db, q, arg1, arg2)
 	if err != nil {
 		return shared.ImageAlias{}, err
 	}
@@ -774,7 +1134,7 @@ func doAliasGet(d *Daemon, name string, isTrustedClient bool) (shared.ImageAlias
 
 func aliasDelete(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	_, _ = shared.DbExec(d.db, "DELETE FROM images_aliases WHERE name=?", name)
+	_, _ = dbExec(d.db, "DELETE FROM images_aliases WHERE name=?", name)
 
 	return EmptySyncResponse
 }
@@ -794,22 +1154,37 @@ func imageExport(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	path := shared.VarPath("images", imgInfo.Fingerprint)
 	filename := imgInfo.Filename
-
+	imagePath := shared.VarPath("images", imgInfo.Fingerprint)
+	rootfsPath := imagePath + ".rootfs"
 	if filename == "" {
-		_, ext, err := detectCompression(path)
+		_, ext, err := detectCompression(imagePath)
 		if err != nil {
 			ext = ""
 		}
 		filename = fmt.Sprintf("%s%s", fingerprint, ext)
 	}
 
-	headers := map[string]string{
-		"Content-Disposition": fmt.Sprintf("inline;filename=%s", filename),
+	if shared.PathExists(rootfsPath) {
+		files := make([]fileResponseEntry, 2)
+
+		files[0].identifier = "metadata"
+		files[0].path = imagePath
+		files[0].filename = "meta-" + filename
+
+		files[1].identifier = "rootfs"
+		files[1].path = rootfsPath
+		files[1].filename = filename
+
+		return FileResponse(r, files, nil, false)
 	}
 
-	return FileResponse(r, path, filename, headers)
+	files := make([]fileResponseEntry, 1)
+	files[0].identifier = filename
+	files[0].path = imagePath
+	files[0].filename = filename
+
+	return FileResponse(r, files, nil, false)
 }
 
 func imageSecret(d *Daemon, r *http.Request) Response {
