@@ -4,7 +4,11 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"io/ioutil"
+	"mime"
+	"mime/multipart"
 	"os"
+	"path/filepath"
 
 	"github.com/lxc/lxd/shared"
 )
@@ -29,7 +33,7 @@ func (d *Daemon) dbGetimage(fp string) int {
 	id := -1
 	arg1 := []interface{}{fp}
 	arg2 := []interface{}{&id}
-	err := shared.DbQueryRowScan(d.db, q, arg1, arg2)
+	err := dbQueryRowScan(d.db, q, arg1, arg2)
 	if err != nil {
 		return -1
 	}
@@ -79,46 +83,89 @@ func ensureLocalImage(d *Daemon, server, fp string, secret string) error {
 	if err != nil {
 		return err
 	}
-	destName := shared.VarPath("images", fp)
-	if _, err := os.Stat(destName); err == nil {
-		os.Remove(destName)
-	}
 
-	f, err := os.Create(destName)
+	builddir, err := ioutil.TempDir(destDir, "lxd_build_")
 	if err != nil {
 		return err
 	}
-	var wr io.Writer
-	wr = f
 
-	_, err = io.Copy(wr, raw.Body)
-	f.Close()
+	/* remove the builddir when done */
+	defer removeImgWorkdir(d, builddir)
+
+	ctype, ctypeParams, err := mime.ParseMediaType(raw.Header.Get("Content-Type"))
 	if err != nil {
-		os.Remove(destName)
-		return err
+		ctype = "application/octet-stream"
 	}
 
-	q := `INSERT INTO images (fingerprint, filename, size, architecture, creation_date, expiry_date, upload_date) VALUES (?, ?, ?, ?, ?, ?, strftime("%s"))`
+	if ctype == "multipart/form-data" {
+		// Parse the POST data
+		mr := multipart.NewReader(raw.Body, ctypeParams["boundary"])
 
-	result, err := shared.DbExec(d.db, q, fp, info.Filename, info.Size, info.Architecture, info.CreationDate, info.ExpiryDate)
-	if err != nil {
-		os.Remove(destName)
-		return err
-	}
+		// Get the metadata tarball
+		part, err := mr.NextPart()
+		if err != nil {
+			return err
+		}
 
-	id64, err := result.LastInsertId()
-	if err != nil {
-		return err
-	}
-	id := int(id64)
+		if part.FormName() != "metadata" {
+			return fmt.Errorf("Invalid multipart image")
+		}
+
+		destName := filepath.Join(builddir, info.Fingerprint)
+		f, err := os.Create(destName)
+		if err != nil {
+			return err
+		}
+
+		_, err = io.Copy(f, part)
+		f.Close()
+
+		if err != nil {
+			return err
+		}
+
+		// Get the rootfs tarball
+		part, err = mr.NextPart()
+		if err != nil {
+			return err
+		}
+
+		if part.FormName() != "rootfs" {
+			return fmt.Errorf("Invalid multipart image")
+		}
+
+		destName = filepath.Join(builddir, info.Fingerprint+".rootfs")
+		f, err = os.Create(destName)
+		if err != nil {
+			return err
+		}
+
+		_, err = io.Copy(f, part)
+		f.Close()
+
+		if err != nil {
+			return err
+		}
+	} else {
+		destName := filepath.Join(builddir, info.Fingerprint)
+
+		f, err := os.Create(destName)
+		if err != nil {
+			return err
+		}
+
+		_, err = io.Copy(f, raw.Body)
+		f.Close()
 
-	for k, v := range info.Properties {
-		q := `INSERT INTO images_properties (image_id, type, key, value) VALUES (?, 0, ?, ?)`
-		_, err = shared.DbExec(d.db, q, id, k, v)
 		if err != nil {
 			return err
 		}
 	}
 
+	_, err = buildImageFromInfo(d, info, builddir)
+	if err != nil {
+		return err
+	}
+
 	return nil
 }
