@@ -6,6 +6,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"mime/multipart"
 	"net/http"
 	"os"
 
@@ -35,42 +36,89 @@ type syncResponse struct {
   fname: name of the file without path
   headers: any other headers that should be set in the response
 */
-type fileResponse struct {
-	req      *http.Request
-	path     string
-	filename string
-	headers  map[string]string
+
+type fileResponseEntry struct {
+	identifier string
+	path       string
+	filename   string
 }
 
-func FileResponse(r *http.Request, path string, filename string, headers map[string]string) Response {
-	return &fileResponse{r, path, filename, headers}
+type fileResponse struct {
+	req              *http.Request
+	files            []fileResponseEntry
+	headers          map[string]string
+	removeAfterServe bool
+}
+
+func FileResponse(r *http.Request, files []fileResponseEntry, headers map[string]string, removeAfterServe bool) Response {
+	return &fileResponse{r, files, headers, removeAfterServe}
 }
 
 func (r *fileResponse) Render(w http.ResponseWriter) error {
-	w.Header().Set("Content-Type", "application/octet-stream")
-
-	f, err := os.Open(r.path)
-	if err != nil {
-		return err
-	}
-	defer f.Close()
-
-	fi, err := f.Stat()
-	if err != nil {
-		return err
-	}
-
 	if r.headers != nil {
 		for k, v := range r.headers {
 			w.Header().Set(k, v)
 		}
 	}
 
-	http.ServeContent(w, r.req, r.filename, fi.ModTime(), f)
-	return nil
+	// No file, well, it's easy then
+	if len(r.files) == 0 {
+		return nil
+	}
+
+	// For a single file, return it inline
+	if len(r.files) == 1 {
+		w.Header().Set("Content-Type", "application/octet-stream")
+		w.Header().Set("Content-Disposition", fmt.Sprintf("inline;filename=%s", r.files[0].filename))
+
+		f, err := os.Open(r.files[0].path)
+		if err != nil {
+			return err
+		}
+		defer f.Close()
+
+		fi, err := f.Stat()
+		if err != nil {
+			return err
+		}
+
+		http.ServeContent(w, r.req, r.files[0].filename, fi.ModTime(), f)
+		if r.removeAfterServe {
+			os.Remove(r.files[0].filename)
+		}
+
+		return nil
+	}
+
+	// Now the complex multipart answer
+	body := &bytes.Buffer{}
+	mw := multipart.NewWriter(body)
+
+	for _, entry := range r.files {
+		fd, err := os.Open(entry.path)
+		if err != nil {
+			return err
+		}
+		defer fd.Close()
+
+		fw, err := mw.CreateFormFile(entry.identifier, entry.filename)
+		if err != nil {
+			return err
+		}
+
+		_, err = io.Copy(fw, fd)
+		if err != nil {
+			return err
+		}
+	}
+
+	mw.Close()
+	w.Header().Set("Content-Type", mw.FormDataContentType())
+	_, err := io.Copy(w, body)
+	return err
 }
 
-func WriteJson(w http.ResponseWriter, body interface{}) error {
+func WriteJSON(w http.ResponseWriter, body interface{}) error {
 	var output io.Writer
 	var captured *bytes.Buffer
 
@@ -96,7 +144,7 @@ func (r *syncResponse) Render(w http.ResponseWriter) error {
 	}
 
 	resp := resp{Type: lxd.Sync, Status: status.String(), StatusCode: status, Metadata: r.metadata}
-	return WriteJson(w, resp)
+	return WriteJSON(w, resp)
 }
 
 /*
@@ -129,7 +177,6 @@ type asyncResponse struct {
 }
 
 func (r *asyncResponse) Render(w http.ResponseWriter) error {
-
 	op, err := CreateOperation(r.metadata, r.resources, r.run, r.cancel, r.ws)
 	if err != nil {
 		return err
@@ -162,7 +209,7 @@ func (r *asyncResponse) Render(w http.ResponseWriter) error {
 	w.Header().Set("Location", op)
 	w.WriteHeader(202)
 
-	return WriteJson(w, body)
+	return WriteJSON(w, body)
 }
 
 func AsyncResponse(run func() shared.OperationResult, cancel func() error) Response {
@@ -217,7 +264,7 @@ func InternalError(err error) Response {
 }
 
 /*
- * Write the right error message based on err.
+ * SmartError returns the right error message based on err.
  */
 func SmartError(err error) Response {
 	switch err {
